# aggregator.py
#
# Copyright (c) 2015  DFN-CERT Services GmbH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from collections import deque
from datetime import timedelta

from mixmod import dataprocessor
from mixmod.errors import ConfigError


# default maximal time difference of different parts of data in seconds
DEFAULT_MAX_TIME_DIFF_IN_FLOW = 600


# FIXME! Currently not implemented as the original implementation was
# buggy
class Aggregator(dataprocessor.DataProcessor):
    """Aggregate data objects from the same source.

    This processor aggregates data objects that originate from the
    same connection across time slices.

    If a data object has fields we do not know how to aggregate, we
    ignore it.

    """

    CONFIG = {
        "max_time_diff_in_flow": {
            "type": "int",
            "default": DEFAULT_MAX_TIME_DIFF_IN_FLOW
        }
    }

    def __init__(self, max_time_diff_in_flow=DEFAULT_MAX_TIME_DIFF_IN_FLOW, **conf):
        super(Aggregator, self).__init__(**conf)
        self.aggregations = 0
        self.data = deque()
        try:
            self.max_time_diff_in_flow = timedelta(seconds=int(max_time_diff_in_flow))
        except ValueError:
            raise ConfigError("Bad value for max_time_diff_in_flow: {}"
                              .format(max_time_diff_in_flow))

    def put(self, data):
        self.data.append(data)
        # if data['ip_version'] == 4:
        #     key = (data['src_ip_v4'], data['dst_ip_v4'])
        # elif data['ip_version'] == 6:
        #     key = (data['src_ip_v6'], data['dst_ip_v6'])
        # else:
        #     raise DataError('Bad IP version in mixmod data set: {}'
        #                     .format(data['ip_version']))
        # key += (data['src_port'], data['dst_port'], data['ip_protocol_number'])
        # if 'utc_first_packet' not in data:
        #     raise DataError('Missing field in mixmod data: utc_first_packet')
        # if 'utc_last_packet' not in data:
        #     raise DataError('Missing field in mixmod data: utc_last_packet')
        # if key in self.flows:
        #     start2 = data['utc_first_packet']
        #     end2 = data['utc_last_packet']
        #     time_diff = self.max_time_diff_in_flow
        #     for f in self.flows[key]:
        #         start1 = f['utc_first_packet']
        #         end1 = f['utc_last_packet']
        #         if (start2 - end1) > timedelta(0):
        #             time_diff_tmp = start2 - end1
        #             start = start1
        #             end = end2
        #         else:
        #             time_diff_tmp = start1 - end2
        #             start = start2
        #             end = end1
        #         if time_diff_tmp < time_diff:
        #             time_diff = time_diff_tmp
        #     if time_diff < self.max_time_diff_in_flow:
        #         # aggregate:
        #         self.aggregations += 1
        #         # occurrences
        #         try:
        #             occurrences = f['occurrences']
        #         except KeyError:
        #             occurrences = 1
        #         f['occurrences'] = occurrences + 1
        #         # tcp_flags
        #         f['tcp_flags'] = f['tcp_flags'] | data['tcp_flags']
        #         # time
        #         f['utc_first_packet'] = start
        #         f['utc_last_packet'] = end
        #         # byte count
        #         if ('byte_count' in data) and ('byte_count' in f):
        #             f['byte_count'] += data['byte_count']
        #         # packet count
        #         if ('packet_count' in data) and ('packet_count' in f):
        #             f['packet_count'] += data['packet_count']
        #     else:
        #         self.flows[key].append(data)
        #         self.mismatches_time += 1
        # else:
        #     self.flows[key] = list()
        #     self.flows[key].append(data)
        #     self.mismatches_prot += 1

    def get(self):
        if self.data:
            return self.data.popleft()
        else:
            return None

    def close(self):
        pass
