# csvexporter.py
#
# Copyright (c) 2015  DFN-CERT Services GmbH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import csv

from mixmod.exporter import FileExporter


# Default fields that are expored to CSV data (can be changed by configuration)
DEFAULT_EXPORT_FIELDS = [
    'src_ip_v4',
    'dst_ip_v4',
    'src_ip_v6',
    'dst_ip_v6',
    'src_port',
    'dst_port',
    'src_as',
    'dst_as',
    'tcp_flags',
    'ip_protocol_number',
    'type_of_service',
    'ip_version',
    'snmp_index_in',
    'snmp_index_out',
    'direction',
    'utc_first_packet',
    'utc_last_packet',
    'packet_count',
    'byte_count',
    'tlp'
]


class CSVExporter(FileExporter):
    """Export data objects to CSV."""

    CONFIG = {
        "export_fields": {
            "type": "string",
            "default": ", ".join(DEFAULT_EXPORT_FIELDS)
        }
    }

    def __init__(self, fobj, export_fields=DEFAULT_EXPORT_FIELDS, **conf):
        super(CSVExporter, self).__init__(fobj, **conf)
        if isinstance(export_fields, basestring):
            self.export_fields = [field.strip() for field in export_fields.split(",")]
        else:
            self.export_fields = export_fields
        self.csv_writer = csv.writer(fobj)
        self.csv_writer.writerow(self.export_fields)

    def put(self, data):
        self.csv_writer.writerow([unicode(data.get(field, ""))
                                  for field in self.export_fields])
