import datetime
import json
import os
import StringIO
import unittest

import netaddr

from mixmod.modules import acdcjsonexporter
from mixmod.tests.utils import BASE_DIR

SCHEMA_DIR = os.path.join(BASE_DIR, "tests/data/schema")


class TestACDCJSONExporter(unittest.TestCase):
    def test_should_dump_acdc_json(self):
        s = StringIO.StringIO()
        with acdcjsonexporter.ACDCJSONExporter(s, schema_dir=SCHEMA_DIR) as e:
            e.put({'report_category': "eu.acdc.attack",
                   'src_ip_v4': netaddr.IPAddress('10.11.12.13'),
                   'src_mode': "plain",
                   'timestamp': datetime.datetime(2015, 6, 17, 11, 00),
                   'reported_at': datetime.datetime(2015, 6, 17, 11, 00),
                   'confidence_level': 1,
                   'ip_protocol_number': 4,
                   'ip_version': 4})

        actual = json.loads(s.getvalue())
        expected = [{u'ip_protocol_number': 4,
                     u'timestamp': u'2015-06-17T11:00:00Z',
                     u'source_key': u'ip',
                     u'report_category': u'eu.acdc.attack',
                     u'confidence_level': 1,
                     u'reported_at': u'2015-06-17T11:00:00Z',
                     u'version': 1,
                     u'src_mode': u'plain',
                     u'report_type': u'This dataset has been automatically generated by Mixmod',
                     u'ip_version': 4,
                     u'source_value': u'10.11.12.13',
                     u'src_ip_v4': u'10.11.12.13',
                     u'report_subcategory': u'other'}]
        self.assertEqual(expected, actual)
