import json
import os
import StringIO
import unittest

from mixmod.modules import acdcjsonimporter
from mixmod.tests.utils import BASE_DIR

SCHEMA_DIR = os.path.join(BASE_DIR, "tests/data/schema")


class TestACDCJSONImporter(unittest.TestCase):
    def test_should_read_nfcap_files(self):
        self.maxDiff = None
        s = StringIO.StringIO()
        expected = {
            u'ip_protocol_number': 4,
            u'timestamp': u'2015-06-17T11:00:00Z',
            u'source_key': u'ip',
            u'report_category': u'eu.acdc.attack',
            u'confidence_level': 1,
            u'reported_at': u'2015-06-17T11:00:00Z',
            u'version': 1,
            u'src_mode': u'plain',
            u'report_type': u'This dataset has been automatically generated by Mixmod',
            u'ip_version': 4,
            u'source_value': u'10.11.12.13',
            u'src_ip_v4': u'10.11.12.13',
            u'report_subcategory': u'other'}
        json.dump(expected, s)
        s.seek(0)
        i = acdcjsonimporter.ACDCJSONImporter(s, schema_dir=SCHEMA_DIR)
        actual = next(iter(i))

        expected["reported_at"] = acdcjsonimporter.to_datetime(expected["reported_at"])
        expected["src_ip_v4"] = acdcjsonimporter.to_ipaddr(expected["src_ip_v4"])
        expected["utc_first_packet"] = acdcjsonimporter.to_datetime(expected["timestamp"])
        expected["timestamp"] = acdcjsonimporter.to_datetime(expected["timestamp"])

        self.assertEqual(expected, actual)
