import unittest

from mixmod import dataprocessor


class DataProcessorTest(dataprocessor.DataProcessor):
    def put(self, data):
        self.conf["data"].append(data)

    def get(self):
        if self.conf["data"]:
            return self.conf["data"].pop()
        else:
            return None

    def close(self):
        self.conf["closed"] = True


class TestDataProcessor(unittest.TestCase):
    def test_should_act_as_context_manager(self):
        with DataProcessorTest(data=[1, 2, 3]) as i:
            actual = [i.get(), i.get(), i.get()]
            self.assertEqual([3, 2, 1], actual)
            self.assertIsNone(i.get())

        self.assertTrue(i.conf["closed"])

    def test_should_act_as_iterator(self):
        with DataProcessorTest(data=[1, 2, 3]) as i:
            actual = list(i)

        self.assertEqual([3, 2, 1], actual)
