import os
import unittest

import netaddr

from mixmod.modules import filter
from mixmod.tests import utils

GEOIP4_DAT = os.path.join(utils.BASE_DIR, "tests/data/geoip/GeoIP.dat")
GEOIP6_DAT = os.path.join(utils.BASE_DIR, "tests/data/geoip/GeoIPv6.dat")


class TestFilter(unittest.TestCase):
    def setUp(self):
        self.filter = filter.Filter("dst ip 10.11.12.13")

    def test_should_keep_matching_data(self):
        expected = {"dst_ip_v4": netaddr.IPAddress("10.11.12.13"),
                    "ip_version": 4}
        self.filter.put(expected)
        actual = self.filter.get()

        self.assertEqual(expected, actual)

    def test_should_drop_nonmatching_data(self):
        expected = {"dst_ip_v4": netaddr.IPAddress("10.11.12.14"),
                    "ip_version": 4}
        self.filter.put(expected)
        actual = self.filter.get()

        self.assertIsNone(actual)

    def test_should_match_geoip4_data(self):
        expected = {"dst_ip_v4": netaddr.IPAddress("216.58.209.99"),
                    "ip_version": 4}
        f = filter.Filter("dst country US", geoip_db_file=GEOIP4_DAT)
        f.put(expected)
        actual = f.get()

        self.assertEqual(expected, actual)

    def test_should_match_geoip6_data(self):
        expected = {"dst_ip_v6": netaddr.IPAddress("2a00:1450:400f:805::2003"),
                    "ip_version": 6}
        f = filter.Filter("dst country IE", geoipv6_db_file=GEOIP6_DAT)
        f.put(expected)
        actual = f.get()

        self.assertEqual(expected, actual)
