import datetime
import os
import unittest

from mixmod.modules import netflow9importer
from mixmod.tests.utils import BASE_DIR
from netaddr import IPAddress


NF9_FILE = os.path.join(BASE_DIR, "tests/data/test.nf9")


class TestNetflow9Importer(unittest.TestCase):
    def test_should_read_netflow9_files(self):
        with open(NF9_FILE) as f:
            importer = netflow9importer.Netflow9Importer(f)
            actual = next(iter(importer))

        self.maxDiff = None

        expected = {'tcp_flags': 20,
                    'direction': 0,
                    'type_of_service': 0,
                    'src_port': 57342,
                    'ip_protocol_number': 6,
                    'byte_count': 40,
                    'snmp_index_in': 0,
                    'dst_ip_v4': IPAddress('141.9.229.180'),
                    'snmp_index_out': 0,
                    'original_format': 'nf9',
                    'packet_count': 1,
                    'ip_version': 4,
                    'dst_port': 445,
                    'src_ip_v4': IPAddress('106.212.40.91'),
                    'utc_first_packet': datetime.datetime(2015, 6, 18, 13, 1, 21, 19000),
                    'utc_last_packet': datetime.datetime(2015, 6, 18, 13, 1, 21, 19000)}

        self.assertEqual(expected, actual)
