import datetime
import os
import unittest

from mixmod.modules import nfcapimporter
from mixmod.tests.utils import BASE_DIR
from netaddr import IPAddress


NFCAP_FILE = os.path.join(BASE_DIR, "tests/data/test.nfcap-20150616-144848")


class TestNFCapImporter(unittest.TestCase):
    def test_should_read_nfcap_files(self):
        with open(NFCAP_FILE) as f:
            i = nfcapimporter.NFCapImporter(f)
            actual = next(iter(i))

        expected = {'tcp_flags': 0,
                    'utc_first_packet': datetime.datetime(2015, 6, 16, 12, 44, 48, 675000),
                    'src_ip_v4': IPAddress('223.47.29.86'),
                    'original_format': 'nfc',
                    'in_src_mac': None,
                    'mpls_label_8': None,
                    'aggrflows': None,
                    'mpls_label_2': None,
                    'mpls_label_3': None,
                    'mpls_label_1': None,
                    'mpls_label_6': None,
                    'mpls_label_7': None,
                    'mpls_label_4': None,
                    'mpls_label_5': None,
                    'dstmask': None,
                    'src_port': 53,
                    'numoutbyts': None,
                    'byte_count': 103,
                    'tos': None,
                    'type_of_service': 0,
                    'in_dst_mac': None,
                    'srcmask': None,
                    'srcvlan': None,
                    'mpls_label_10': None,
                    'out_dst_mac': None,
                    'engine_type': None,
                    'fwd_status': 0,
                    'engine_id': None,
                    'dst_ip_v4': IPAddress('194.37.140.78'),
                    'snmp_index_in': 0,
                    'mpls_label_9': None,
                    'numoutpackets': None,
                    'router_ip_version': None,
                    'ip_version': 4,
                    'utc_last_packet': datetime.datetime(2015, 6, 16, 12, 48, 48, 687000),
                    'router_ip': None,
                    'next_bgp_ip_version': None,
                    'fill_e25': None,
                    'src_tos': 0,
                    'ident': '',
                    'next_bgp_ip': None,
                    'snmp_index_out': 0,
                    'packet_count': 1,
                    'ip_protocol_number': 17,
                    'next_hop_ip': None,
                    'next_hop_ip_version': None,
                    'dst_port': 9667,
                    'dstvlan': None,
                    'out_src_mac': None,
                    'dir': None}

        self.assertEqual(expected, actual)
