import datetime
import os
import unittest

from netaddr import IPAddress

from mixmod.modules import pcapimporter
from mixmod.tests.utils import BASE_DIR


PCAP_FILE = os.path.join(BASE_DIR, "tests/data/test.pcap")


class TestPCapImporter(unittest.TestCase):
    def test_should_import_simple_pcap_file(self):
        with open(PCAP_FILE) as f:
            pcap = pcapimporter.PCapImporter(f)
            actual = pcap.get()

        expected = {
            'tcp_flags': 0,
            'direction': 0,
            'utc_last_packet': datetime.datetime(2015, 6, 18, 6, 34, 43, 158000),
            'ip_protocol_number': 1,
            'dst_ip_v4': IPAddress('127.0.0.1'),
            'original_format': 'pcap',
            'byte_count': 98,
            'packet_count': 1,
            'utc_first_packet': datetime.datetime(2015, 6, 18, 6, 34, 43, 158000),
            'icmp_type_ipv4': 8,
            'ip_version': 4,
            'icmp_code_ipv4': 0,
            'src_ip_v4': IPAddress('127.0.0.1')
        }

        self.assertEqual(expected, actual)
