#!/usr/bin/env python

import json
import sys

from mixmod import importer, exporter, dataprocessor
from mixmod.config import MIXMOD_CLASSES


def main():
    mixmod_modules = []
    for name, cls in MIXMOD_CLASSES.items():
        type_ = get_type(cls)
        module = {
            "type": type_,
            "title": name,
            "info": getattr(cls, "__doc__", ""),
            "endpoints": get_endpoints(type_),
            "config": {
                "section": name,
                "items": {
                    name: {
                        "info": "Undocumented",
                        "type": conf["type"],
                        "default": conf["default"]
                    }
                    for name, conf in cls.CONFIG.items()
                }
            }
        }
        mixmod_modules.append(module)

    sys.stdout.write("var mixmod_modules = ")
    json.dump(mixmod_modules, sys.stdout)


def get_type(cls):
    if issubclass(cls, importer.Importer):
        return "importer"
    elif issubclass(cls, exporter.Exporter):
        return "exporter"
    elif issubclass(cls, dataprocessor.DataProcessor):
        return "dataprocessor"
    else:
        raise RuntimeError("Unknown type for class {}".format(cls.__name__))


def get_endpoints(type_):
    if type_ == "importer":
        return ["source"]
    elif type_ == "exporter":
        return ["target"]
    elif type_ == "dataprocessor":
        return ["target", "source"]
    else:
        raise RuntimeError("Unknown type {}".format(type_))


if __name__ == '__main__':
    main()
