import pytest
from guigen import fill_guiinfo
from guigen import import_modules
from guigen import list_modules


def test_complement_guiinfo():
    valid_import = {"type": "import", "title": "AweSomeImporter", "endpoints": ['source'], "config": {"section": "AweSomeImporter"}}
    valid_export = valid_import.copy()
    fancy_export = valid_export.copy()
    valid_dataprocessor = valid_import.copy()

    valid_export.update({"type": "export", "endpoints": ['target']})
    valid_dataprocessor.update({"type": "intermediate", "endpoints": ['source', 'target']})
    fancy_export.update({"title": "AnotherFooBarsTheDust"})

    test_data = [
        # add missing data
        [{k: v for k, v in valid_import.iteritems() if k != "title"}, valid_import],
        [{k: v for k, v in valid_import.iteritems() if k != "endpoints"}, valid_import],
        [{k: v for k, v in valid_export.iteritems() if k != "endpoints"}, valid_export],
        [{k: v for k, v in valid_dataprocessor.iteritems() if k != "endpoints"}, valid_dataprocessor],

        # don't destroy given data
        [valid_import, valid_import],
        [valid_export, valid_export],
        [valid_dataprocessor, valid_dataprocessor],
        [fancy_export, fancy_export],
    ]

    for input, result in test_data:
        oldinput = input.copy()

        # check if it filly correctly
        assert fill_guiinfo(input) == result

        # check if that it does not modify paramater
        assert input == oldinput

    # throw exception when missing mandatory fields
    with pytest.raises(ValueError) as excinfo:
        fill_guiinfo({})
    assert 'Missing mandatory field' in str(excinfo.value)


def test_list_modules():
    result = list_modules("./tests/fixtures")
    assert set(result) == set(['info1', 'info2', 'info3'])


def test_import_modules():
    result = import_modules("./tests/fixtures")
    assert 3 == len(result)
    for entry in result:
        assert "noinfo" not in entry.path
        assert entry.module_name.lower() in entry.path.lower()
        assert isinstance(entry.guiinfo, dict)
